﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Trolleybus.DrawingObjects;
using Trolleybus.Generics;
using Trolleybus.MovementStrategy;

namespace Trolleybus
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawingBus
    /// </summary>
    public partial class FormBusesCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly BusesGenericCollection<DrawingBus, DrawingObjectBus> _buses;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBusesCollection()
        {
            InitializeComponent();
            _buses = new BusesGenericCollection<DrawingBus, DrawingObjectBus>(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddBus_Click(object sender, EventArgs e)
        {
            FormTrolleybus form = new FormTrolleybus();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (_buses + form.SelectedBus != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = _buses.ShowBuses();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBus_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            String pos_string = maskedTextBoxNumber.Text;
            if (pos_string == "")
            {
                pos_string = "0";
            }
            int pos = Convert.ToInt32(pos_string);
            if (_buses - pos)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = _buses.ShowBuses();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _buses.ShowBuses();
        }

        private void InitializeComponent()
        {
            pictureBoxCollection = new PictureBox();
            panelTools = new Panel();
            buttonRefreshCollection = new Button();
            maskedTextBoxNumber = new MaskedTextBox();
            buttonRemoveBus = new Button();
            buttonAddBus = new Button();
            ((System.ComponentModel.ISupportInitialize)pictureBoxCollection).BeginInit();
            panelTools.SuspendLayout();
            SuspendLayout();
            // 
            // pictureBoxCollection
            // 
            pictureBoxCollection.Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right;
            pictureBoxCollection.Location = new Point(0, 0);
            pictureBoxCollection.Name = "pictureBoxCollection";
            pictureBoxCollection.Size = new Size(670, 453);
            pictureBoxCollection.TabIndex = 0;
            pictureBoxCollection.TabStop = false;
            // 
            // panelTools
            // 
            panelTools.Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Right;
            panelTools.Controls.Add(buttonRefreshCollection);
            panelTools.Controls.Add(maskedTextBoxNumber);
            panelTools.Controls.Add(buttonRemoveBus);
            panelTools.Controls.Add(buttonAddBus);
            panelTools.Location = new Point(682, 0);
            panelTools.Name = "panelTools";
            panelTools.Size = new Size(200, 453);
            panelTools.TabIndex = 1;
            // 
            // buttonRefreshCollection
            // 
            buttonRefreshCollection.Location = new Point(17, 190);
            buttonRefreshCollection.Name = "buttonRefreshCollection";
            buttonRefreshCollection.Size = new Size(170, 40);
            buttonRefreshCollection.TabIndex = 3;
            buttonRefreshCollection.Text = "Обновить коллекцию";
            buttonRefreshCollection.UseVisualStyleBackColor = true;
            buttonRefreshCollection.Click += ButtonRefreshCollection_Click;
            // 
            // maskedTextBoxNumber
            // 
            maskedTextBoxNumber.Location = new Point(39, 76);
            maskedTextBoxNumber.Mask = "00";
            maskedTextBoxNumber.Name = "maskedTextBoxNumber";
            maskedTextBoxNumber.Size = new Size(125, 27);
            maskedTextBoxNumber.TabIndex = 2;
            maskedTextBoxNumber.ValidatingType = typeof(int);
            // 
            // buttonRemoveBus
            // 
            buttonRemoveBus.Location = new Point(17, 125);
            buttonRemoveBus.Name = "buttonRemoveBus";
            buttonRemoveBus.Size = new Size(170, 40);
            buttonRemoveBus.TabIndex = 1;
            buttonRemoveBus.Text = "Удалить автобус";
            buttonRemoveBus.UseVisualStyleBackColor = true;
            buttonRemoveBus.Click += ButtonRemoveBus_Click;
            // 
            // buttonAddBus
            // 
            buttonAddBus.Location = new Point(17, 12);
            buttonAddBus.Name = "buttonAddBus";
            buttonAddBus.Size = new Size(170, 40);
            buttonAddBus.TabIndex = 0;
            buttonAddBus.Text = "Добавить автобус";
            buttonAddBus.UseVisualStyleBackColor = true;
            buttonAddBus.Click += ButtonAddBus_Click;
            // 
            // FormBusCollection
            // 
            ClientSize = new Size(882, 453);
            Controls.Add(panelTools);
            Controls.Add(pictureBoxCollection);
            Name = "FormBusCollection";
            Text = "Набор автобусов";
            ((System.ComponentModel.ISupportInitialize)pictureBoxCollection).EndInit();
            panelTools.ResumeLayout(false);
            panelTools.PerformLayout();
            ResumeLayout(false);
        }

        private PictureBox pictureBoxCollection;
        private Panel panelTools;
        private Button buttonRefreshCollection;
        private MaskedTextBox maskedTextBoxNumber;
        private Button buttonRemoveBus;
        private Button buttonAddBus;
    }
}
