﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Trolleybus.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T?[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _places = new T?[count];
        }
        /// <summary>
        /// Добавление объекта в начало набора
        /// </summary>
        /// <param name="bus">Добавляемый автобус</param>
        /// <returns></returns>
        public int Insert(T bus)
        {
            if (_places[0] == null)
            {
                _places[0] = bus;
                //0 в данном случае - индекс в массиве, вставка прошла успешно
                return 0;
            }
            else
            {
                int index = 0;
                while (_places[index] != null)
                {
                    index++;
                    if (index >= Count)
                    {
                        //места в массиве нет, ни по какому индексу вставить нельзя
                        return -1;
                    }
                }
                //cдвиг элементов
                for (int i = index; i > 0; i--)
                {
                    _places[i] = _places[i - 1];

                }
                _places[0] = bus;
                //0 в данном случае - индекс в массиве, вставка прошла успешно
                return 0;
            }

        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="bus">Добавляемый автобус</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T bus, int position)
        {
            if (position >= Count || position < 0)
            {
                //индекс неверный, значит вставить по индексу нельзя
                return -1;
            }
            if (_places[position] == null)
            {
                _places[position] = bus;
            }
            else
            {
                //проверка, что в массиве после вставляемого эл-а есть место
                int index = position;
                while (_places[index] != null)
                {
                    index++;
                    if (index >= Count)
                    {
                        //места в массиве нет, ни по какому индексу вставить нельзя
                        return -1;
                    }
                }
                //сдвиг элементов
                for (int i = index; i > position; i--)
                {
                    _places[i] = _places[i - 1];

                }
                //вставка по позиции
                _places[position] = bus;

            }
            //индекс в массиве, по которому вставили, вставка прошла успешно
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (position >= Count || position < 0)
            {
                return false;
            }
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? Get(int position)
        {
            if (position >= Count || position < 0)
            {
                return null;
            }
            return _places[position];
        }
    }
}
